/*************************************************************************/
/* Program  : AnaMin.c  v4.0  (analog clock with minutes only)           */
/* Compiler : Borland Turbo C++ v3.1 Visual Edition for MS-Windows 3.1   */
/* Date     : July 1st, 1994                                             */
/* Author   : Josef D. Gonko                                             */
/* Abstract : Read .DOC file!                                            */
/*************************************************************************/

// includes and defines
#include <windows.h>                 // standard include file for Win apps
#include <stdio.h>                   // for fopen, fread, fwrite, fclose
#include <stdlib.h>
#include <string.h>                  // to use ltoa function
#include <dos.h>                     // standard file to get system time
#include <math.h>                    // for cos and sin functions
#define  TIMER_15   1                // first timer set for 15 seconds
#define  TIMER_SEC  2                // second timer set for 1 second
#define  XSIZE      54               // changeable
#define  YSIZE      53               // changeable

// global variables
int           xcoor = 10;            // used to store window's position &
int           ycoor = 10;            // init for safety prior to load
unsigned int  SystemResources = 0;   // holds SR for bar #1
unsigned int  GDIResources = 0;      // holds GDI for bar #2
unsigned int  UserResources = 0;     // holds User for bar #3
long          DiskFree = 0;          // holds HD space in Kb
long          DiskFreePrev = 0;      // holds (HD space - 10s) in Kb
DWORD         TotalRAM = 0;          // holds RAM space in Kb
DWORD         TotalRAMPrev = 0;      // holds (RAM space - 10s) in Kb
char          HDcolor = 1;           // HD (black:1, red:2, blue:3) 
char          RAMcolor = 1;          // RAM (black:1, red:2, blue:3)

// prototypes
long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG);
void ClockPaint (HDC, HPEN, HPEN, HFONT);  // paints the window
char GetDatInfo (void);                    // will load (xcoor,ycoor)
void SaveDatInfo (int x, int y);     // saves (xcoor,ycoor) on exit
void Resources (void);               // get & display system resources


// functions and procedures
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow) {
  static char  szAppName[] = "AnaMin v4.0";
  HWND         hwnd;
  MSG          msg;
  WNDCLASS     wndclass;
  static char  IniRetrieved = 0;

  if (!hPrevInstance) {
    wndclass.style         = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc   = WndProc;
    wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = 0;
    wndclass.hInstance     = hInstance;
    wndclass.hIcon         = LoadIcon (hInstance, "ICON_1");
    wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
    wndclass.hbrBackground = GetStockObject (LTGRAY_BRUSH);
    wndclass.lpszMenuName  = NULL;
    wndclass.lpszClassName = szAppName;
    RegisterClass (&wndclass);
   }
  if (!IniRetrieved) IniRetrieved = GetDatInfo (); // done only once
  hwnd = CreateWindowEx (WS_EX_TOPMOST,          // extended window paramtr
                         szAppName,              // window class name
		         szAppName,              // window caption bar title
		         WS_POPUP,               // window style
		         xcoor,                  // loaded x position
		         ycoor,                  // loaded y position
		         XSIZE,                  // x size
		         YSIZE,                  // y size
		         NULL,                   // parent window handle
		         NULL,                   // window menu handle
		         hInstance,              // program instance handle
		         NULL);                  // creation parameters
  if (!SetTimer (hwnd, TIMER_15, 15000, NULL)) {    // set 60s timer
    MessageBox (hwnd, "Need 3 timers!", szAppName, MB_ICONEXCLAMATION | MB_OK);
    return FALSE;                              // false ends program
   }
  if (!SetTimer (hwnd, TIMER_SEC, 1000, NULL)) {
    MessageBox (hwnd, "Need 3 timers!", szAppName, MB_ICONEXCLAMATION | MB_OK);
    return FALSE;
   }
  ShowWindow (hwnd, SW_SHOWNOACTIVATE);        // run app but no init focus
  SetWindowLong(hwnd, GWL_EXSTYLE,             // sets top-most style
    GetWindowLong(hwnd, GWL_EXSTYLE) | WS_EX_TOPMOST);
  UpdateWindow (hwnd);                         // put it on the screen
  while (GetMessage (&msg, NULL, 0, 0)) {      // message dispatching method
    TranslateMessage (&msg);
    DispatchMessage (&msg);
   }
  return msg.wParam;
} // end proc WinMain


long FAR PASCAL _export WndProc (HWND hwnd, UINT message,
                                 UINT wParam, LONG lParam) {
  HDC             hdc;
  PAINTSTRUCT     ps;
  RECT            rc;  // struct used to get program's screen coordinates
  static HPEN     hBlackPen, hWhitePen;  // drawing pens
  static LOGFONT  logfont;
  static HFONT    hFont8;           // handle to store height 8 font
  static HWND     hSavedhWnd;    // handle to window with previous focus

  switch (message) {
    case WM_CREATE :
      logfont.lfHeight = 8;               // set font request for 14
      hFont8 = CreateFontIndirect(&logfont);   // get above said font
      hBlackPen = CreatePen (PS_SOLID, 1, RGB (0, 0, 0)); // gimme pens!!
      hWhitePen = CreatePen (PS_SOLID, 1, RGB (255, 255, 255));
      return 0;
    case WM_NCHITTEST :
      return (long) HTCAPTION;            // trick Windows into moving app
    case WM_TIMER :                      
      switch (wParam) {
	case TIMER_15 :
          InvalidateRect (hwnd, NULL, TRUE);  // timer msg begets WM_PAINT msg
          break;
	case TIMER_SEC :
	  hSavedhWnd = GetFocus ();           // need to get this a LOT
	  break;
       }
      return 0;                           
    case WM_MOVE :
      SetFocus(hSavedhWnd);               // BINGO!  Returns focus
      return 0;
    case WM_PAINT :                 
      hdc = BeginPaint (hwnd, &ps);
      Resources();                            // load up resources
      ClockPaint (hdc, hBlackPen, hWhitePen, hFont8); // paints window
      EndPaint (hwnd, &ps);
      return 0;
    case WM_DESTROY :                     // program is terminating
      KillTimer (hwnd, TIMER_15);         // kills timer with ID=1
      KillTimer (hwnd, TIMER_SEC);        // kills timer with ID=2
      DeleteObject (hBlackPen);           // give back loaned object
      DeleteObject (hWhitePen);           // give back loaned object
      DeleteObject (hFont8);              // give back font
      GetWindowRect (hwnd, &rc);          // fill wndw coord struct
      SaveDatInfo (rc.left, rc.top);      // save (xcoor,ycoor)
      PostQuitMessage (0);
      return 0;
   }
  return DefWindowProc (hwnd, message, wParam, lParam);
} // end proc WndProc


void Resources (void) {
  struct  diskfree_t free;
  UINT    dummy;            // var does nothing, just req'd parameter

  // windows resources
  SystemResources = GetFreeSystemResources (GFSR_SYSTEMRESOURCES);
  GDIResources = GetFreeSystemResources (GFSR_GDIRESOURCES);
  UserResources = GetFreeSystemResources (GFSR_USERRESOURCES);
  // hard disk sizing
  _dos_getdiskfree (0, &free);                 // fill free structure
  DiskFree = (long) ((long) free.avail_clusters *
	             (long) free.bytes_per_sector *
		     (long) free.sectors_per_cluster) / 1024;
  if (DiskFreePrev == DiskFree)      HDcolor = 1;  // no change
  else if (DiskFreePrev > DiskFree)  HDcolor = 2;  // minus HD space
  else if (DiskFreePrev < DiskFree)  HDcolor = 3;  // plus HD space 
  DiskFreePrev = DiskFree;
  // memory sizing
  TotalRAM = (long) (GetFreeSpace (dummy) / 1024);
  if (TotalRAMPrev == TotalRAM)      RAMcolor = 1; // no change
  else if (TotalRAMPrev > TotalRAM)  RAMcolor = 2; // minus RAM space
  else if (TotalRAMPrev < TotalRAM)  RAMcolor = 3; // plus RAM space
  TotalRAMPrev = TotalRAM;
}


char GetDatInfo (void) {
  FILE *fd;

  if ((fd = fopen ("ANAMIN.DAT", "r")) != NULL) {
    fread (&xcoor, sizeof (int), 1, fd);
    fread (&ycoor, sizeof (int), 1, fd);
    fclose (fd);
   }
  else
    SaveDatInfo (10, 10);   // only occurs when ANAMIN.DAT D.N.E.
  return 1;
}                           // and saves defaults to create file


void SaveDatInfo (int x, int y) {
  FILE *fd;                 // saves last position of clock upon exit

  if ((fd = fopen ("ANAMIN.DAT", "w")) != NULL) {
    fwrite (&x, sizeof (int), 1, fd);
    fwrite (&y, sizeof (int), 1, fd);
    fclose (fd);
   }
}


void ClockPaint (HDC hdc, HPEN hBlackPen, HPEN hWhitePen,HFONT hFont8) {
  static struct  time t;            // static for speed increase??
  static char    DS[8];             // shouldn't exceed 1234567 Kb
  static char    RS[6];             // shouldn't exceed 99999 Kb

  gettime (&t);                     // fill up time=t structure
  // draw 3D raised-up effect
  SelectObject (hdc, hWhitePen);
    // top and left lines condensed to 3 fcn calls per 2 lines drawn
  MoveTo (hdc, XSIZE, 0);
    LineTo (hdc, 0, 0);
    LineTo (hdc, 0, YSIZE);
  MoveTo (hdc, 1, YSIZE-1);
    LineTo (hdc, 1, 1);
    LineTo (hdc, XSIZE-1, 1);
  SelectObject (hdc, hBlackPen);
  MoveTo (hdc, 1, YSIZE-1);
    LineTo (hdc, XSIZE, YSIZE-1);   // bottom line
  MoveTo (hdc, XSIZE-1, 1);
    LineTo (hdc, XSIZE-1, YSIZE-1); // right line
  MoveTo (hdc, 2, YSIZE-2);
    LineTo (hdc, XSIZE-1, YSIZE-2); // bottom line #2
  MoveTo (hdc, XSIZE-2, 2);
    LineTo (hdc, XSIZE-2, YSIZE-2); // right line #2
  // draw tick lines
  /* for (i=0; i<360; i+=30) { // this is an adjustable version
    MoveTo (hdc, (20.0 + sin (i * 0.01745329252) * 17.0),
		 (20.0 - cos (i * 0.01745329252) * 17.0));
    LineTo (hdc, (20.0 + sin (i * 0.01745329252) * 19.0),
		 (20.0 - cos (i * 0.01745329252) * 19.0));
   } */
  MoveTo (hdc, 22, 5);   LineTo (hdc, 22, 3);   // 0deg  12 noon
  MoveTo (hdc, 30, 7);   LineTo (hdc, 31, 5);   // 30d    1 pm
  MoveTo (hdc, 36, 13);  LineTo (hdc, 38, 12);  // 60d    2 pm
  MoveTo (hdc, 39, 22);  LineTo (hdc, 41, 22);  // 90d    3 pm
  MoveTo (hdc, 36, 30);  LineTo (hdc, 38, 32);  // 120d   4 pm
  MoveTo (hdc, 30, 36);  LineTo (hdc, 32, 38);  // 150d   5 pm
  MoveTo (hdc, 22, 39);  LineTo (hdc, 22, 41);  // 180d   6 pm
  MoveTo (hdc, 13, 36);  LineTo (hdc, 11, 38);  // 210d   7 pm
  MoveTo (hdc, 7, 30);   LineTo (hdc, 5, 32);   // 240d   8 pm
  MoveTo (hdc, 5, 22);   LineTo (hdc, 3, 22);   // 270d   9 pm
  MoveTo (hdc, 7, 13);   LineTo (hdc, 5, 12);   // 300d  10 pm
  MoveTo (hdc, 13, 7);   LineTo (hdc, 12, 5);   // 330d  11 pm
  // draw clock hands
  MoveTo (hdc, 22, 22);
  LineTo (hdc, (int)(22.0 + sin ((t.ti_hour * 30.0 + 0.5 * t.ti_min) * 0.01745329252) * 10.0),
	       (int)(22.0 - cos ((t.ti_hour * 30.0 + 0.5 * t.ti_min) * 0.01745329252) * 10.0));
      // the 22.0's are the CENTER of the clock
  MoveTo (hdc, 22, 22);
  LineTo (hdc, (int)(22.0 + sin (t.ti_min * 0.1047197551) * 15.0),
	       (int)(22.0 - cos (t.ti_min * 0.1047197551) * 15.0));
  // draw resources lines
    // first 3 return percentage values
  MoveTo (hdc, 45, 40 - (int)(35 * SystemResources / 100));
    LineTo (hdc, 45, 40);                           // SystemResources
  MoveTo (hdc, 47, 40 - (int)(35 * GDIResources / 100));
    LineTo (hdc, 47, 40);                           // GDIResources
  MoveTo (hdc, 49, 40 - (int)(35 * UserResources / 100));
    LineTo (hdc, 49, 40);                           // UserResources
  // places HD and RAM free strings
  SelectObject (hdc, hFont8);
  SetBkColor (hdc, RGB (192, 192, 192));
  ltoa (DiskFree, DS, 10);
  switch (HDcolor) {
    case 1 : SetTextColor (hdc, RGB (0, 0, 0));
             TextOut (hdc, 3, 42, DS, strlen(DS));
             break;
    case 2 : SetTextColor (hdc, RGB (255, 0, 0));
             TextOut (hdc, 3, 42, DS, strlen(DS));
             break;
    case 3 : SetTextColor (hdc, RGB (0, 0, 255));
             TextOut (hdc, 3, 42, DS, strlen(DS));
             break;
   }
  ltoa (TotalRAM, RS, 10);
  switch (RAMcolor) {
    case 1 : SetTextColor (hdc, RGB (0, 0, 0));
	     TextOut (hdc, 31, 42, RS, strlen(RS));
	     break;
    case 2 : SetTextColor (hdc, RGB (255, 0, 0));
	     TextOut (hdc, 31, 42, RS, strlen(RS));
	     break;
    case 3 : SetTextColor (hdc, RGB (0, 0, 255));
	     TextOut (hdc, 31, 42, RS, strlen(RS));
	     break;
   }
}

